package FDN::Map::Base;

use strict;
use mapscript;
use Exporter;
use FDN::Common::Logger;

our @ISA = ( 'Exporter' ) ; 

our @EXPORT = qw (
	$colors
	new_color
	new_map
	map_use_symbol
	map_draw_to
	);

our @EXPORT_OK = qw ( );


our $debug = 1;

sub __debug($) {
	my ( $msg ) = @_;
	return if ( ! $debug );
	debug $msg;
}

our $colors = {};

sub new_color($$$$) {
	my ( $name, $r, $g, $b ) = @_;
	$colors->{$name} =  new mapscript::colorObj($r,$g,$b);
}

new_color('white',255,255,255);
new_color('light',232,232,232);
new_color('dark',32,32,32);
new_color('black',0,0,0);

new_color('red',255,0,0);
new_color('green',0,255,0);
new_color('blue',0,0,255);


sub new_map($$$$) {
	my ( $output, $x, $y, $name ) = @_;
	__debug "New map";
	my $map = new mapscript::mapObj();
	$map->selectOutputFormat($output);
	$map->{'name'} = $name;
	$map->{'width'} = $x;
	$map->{'height'} = $y;

	my $res = {
		'map' => $map,
		'symbols' => {}
	};
	return $res;
}

my $symbols = {	
	'circle' => {
		'fct'  => \&symbol_circle,
		'sym'  => undef
	},
	'star' => {
		'fct'  => \&symbol_star,
		'sym'  => undef
	}
};


sub symbol_circle() {
	__debug "Creating circle";
	my $sym = new mapscript::symbolObj('circle');
	$sym->{'type'} = $mapscript::MS_SYMBOL_ELLIPSE;
	$sym->{'filled'} = $mapscript::MS_TRUE;
	$sym->{'sizex'} = 5;
	$sym->{'sizey'} = 5; 
	return $sym;
}

sub symbol_star() {
	my $lin = new mapscript::lineObj();
	$lin->add(new mapscript::pointObj(0, .375));
	$lin->add(new mapscript::pointObj(.35, .375));
	$lin->add(new mapscript::pointObj(.5, 0));
	$lin->add(new mapscript::pointObj(.65, .375));
	$lin->add(new mapscript::pointObj(1, .375));
	$lin->add(new mapscript::pointObj(.75, .625));
	$lin->add(new mapscript::pointObj(.875, 1));
	$lin->add(new mapscript::pointObj(.5, .75));
	$lin->add(new mapscript::pointObj(.125, 1));
	$lin->add(new mapscript::pointObj(.25, .625));

	my $sym = new mapscript::symbolObj('star');
	$sym->{'type'} = $mapscript::MS_SYMBOL_VECTOR;
	$sym->{'filled'} = $mapscript::MS_TRUE;
	$sym->setPoints($lin);
	$sym->{'sizex'} = 1;
	$sym->{'sizey'} = 1;
	return $sym;
}

sub init_symbol($) {
	my ( $symbol ) = @_;
	if ( ! defined $symbols->{$symbol}->{'sym'} ) {
		__debug "Creating <$symbol>";
		$symbols->{$symbol}->{'sym'} = &{$symbols->{$symbol}->{'fct'}}();
	}
}

sub map_use_symbol($$) {
	my ( $map, $symbol ) = @_;
	__debug "Map uses <$symbol>";
	if ( ! defined $map->{'symbols'}->{$symbol} ) {
		init_symbol($symbol);
		my $sym = $symbols->{$symbol}->{'sym'};
		$map->{'map'}->{'symbolset'}->appendSymbol($sym);
		$map->{'symbols'}->{$symbol} = $map->{'map'}->{'symbolset'}->index($symbol);
	}
}

sub map_draw_to($$) {
	my ( $map, $file ) = @_;
	__debug "Drawing map to $file";
	my $img = $map->{'map'}->draw();
	$img->save($file);
}


1;
