package FDN::Map::France;

use strict;
use mapscript;
use Exporter;
use FDN::Common::Logger;
use FDN::Common::Modules;
use FDN::Map::Base;

our @ISA = ( 'Exporter' ) ;

our @EXPORT = qw (
	new_map_france
	new_map_dept
	map_add_dept
	map_add_region
	map_add_ville
	);

our @EXPORT_OK = qw (
	$debug
	);


our $debug = 1;

sub __debug ($) {
	my ( $msg ) = @_;
	return if ( ! $debug );
	debug $msg;
}

sub new_map_france($$$$;$) {
	my ( $output, $x, $y, $name, $metropole ) = @_;
	__debug "New map";
	if ( ! defined $metropole ) {
		$metropole = 1;
	}
	my $map = new_map($output, $x, $y, $name);
	$map->{'map'}->setExtent(55638.000000, 6049647.000000, 1242375.000000, 7110524.000000);
	$map->{'France-dept'} = undef;
	$map->{'metropole'} = $metropole;
	return $map;
}

sub __get_dept_extent($$) {
	my ( $map, $dept ) = @_;
	my $layer = new mapscript::layerObj($map->{'map'});
	$layer->{'data'} = '/usr/share/fdn/maps/France/dep_france_dom';
	$layer->open();
}

sub new_map_dept($$$$$) {
	my ( $output, $x, $y, $dept, $name ) = @_;
	__debug "New map departement $dept as $name";
	my $map = new_map($output, $x, $y, $name);
	# j'ai dû commenter cette ligne, qu'était-elle censée faire ?
	# ERROR
	__get_dept_extent($map, $dept);
	$map->{'map'}->setExtent();
	return $map;
}

sub init_dept($) {
	my ( $map ) = @_;
	return if ( defined $map->{'France-dept'} );
	my $dept_layer;
	__debug "Init dept";

	$dept_layer = new mapscript::layerObj($map->{'map'});
	$dept_layer->{'data'} = '/usr/share/fdn/maps/France/dep_france_dom';
	$dept_layer->{'status'} = $mapscript::MS_ON;
	$dept_layer->{'type'} = $mapscript::MS_LAYER_POLYGON;
	$dept_layer->{'name'} = 'fond';
	$dept_layer->{'classitem'} = 'CODE_DEPT';
	#$dept_layer->setFilter('(("[NOM_REGION]" in "LIMOUSIN,ALSACE") OR ("[NOM_DEPT]" in "LOIRE,LOIRET,MANCHE,SARTHE"))');
	if ( $map->{'metropole'} ) {
		$dept_layer->setFilter('((("[CODE_DEPT]" le "95") AND ("[CODE_DEPT]" ne "2A")) AND ("[CODE_DEPT]" ne "2B"))');
	}
	$map->{'France-dept'} = $dept_layer;
}

sub set_color($$$) {
	my ( $class, $color, $outline ) = @_;
	__debug "Set color";
	my $style = new mapscript::styleObj($class);
	$style->{'color'} = $colors->{$color};
	$style->{'outlinecolor'}  = $colors->{$outline};
}

sub map_add_dept($$$;$) {
	my ( $map, $dept, $color, $outline ) = @_;
	__debug "Add dept";
	init_dept($map);
	$outline = 'black' if ( ! defined $outline );

	my $cl = new mapscript::classObj($map->{'France-dept'});
	$cl->setExpression('"'.$dept.'"') if ( defined $dept );
	set_color($cl,$color,$outline);
}

sub map_add_region($$$;$) {
	my ( $map, $region, $color, $outline ) = @_;
	__debug "Add region";
	init_dept($map);
	$outline = 'black' if ( ! defined $outline );
	my $cl = new mapscript::classObj($map->{'France-dept'});
	$cl->setExpression('("[NOM_REGION]" eq "'.$region.'")');
	set_color($cl,$color,$outline);
}

sub map_add_ville($$$;$$$) {
	my ( $map, $zip, $ville, $symbol, $size, $color, $outline ) = @_;
	__debug "Add ville";
	$symbol = 'circle' if ( ! defined $symbol );
	$size = 5 if ( ! defined $size );
	$color = 'red' if ( ! defined $color );
	$outline = 'black' if ( ! defined $outline );
	map_use_symbol($map,$symbol);
	__debug "Symbol $symbol ready";
	
	my $layer = new mapscript::layerObj($map->{'map'});
	$layer->{'type'} = $mapscript::MS_LAYER_POINT;
	$layer->{'status'} = $mapscript::MS_ON;
	$layer->{'name'} = "$zip/$ville";
	$layer->{'template'} = '/dev/null';

	my ( $x, $y ) = search_zip($zip, $ville);

	my $point = new mapscript::pointObj();
	$point->{x} = $x;
	$point->{y} = $y;

	my $line  = new mapscript::lineObj();
	$line->add($point);

	my $shape = new mapscript::shapeObj($mapscript::MS_LAYER_POINT);
	$shape->add($line);

	$layer->addFeature($shape);

	my $cl = new mapscript::classObj($layer);
	$cl->{'name'} = "$zip/$ville";
	$cl->{'template'} = '/dev/null';

	my $style = new mapscript::styleObj($cl);
	$style->{'color'} = $colors->{$color};
	$style->{'outlinecolor'} = $colors->{$outline};
	__debug "Symbol is ".$map->{symbols}->{$symbol};
	$style->{'symbol'} = $map->{symbols}->{$symbol};
	$style->{'size'} = $size;
}

sub search_zip($$) {
	my ( $zip, $ville ) = @_;
	conduse('FDN::Sql::Villes::Basic');
	conduse('FDN::Common::Levenshtein');
	
	my @poss = mapvil_par_zip($zip);
	if ( $#poss == 0 && ! defined $poss[0] ) {
		die "Should be connected to the DB";
	}

	my $dist = 9999;
	$ville = uc($ville);
	if ( $#poss == -1 ) {
		error "NOT FOUND $zip / $ville\n";
		return undef;
	}
	my $best;
	for my $vil ( @poss ) {
		my $nom = uc($vil->{'MAPVIL_NOM'});
		my $tmp = levenshtein($ville,$nom);
		if ( $tmp < $dist ) {
			$dist = $tmp;
			$best = $vil;
		}
	}
	return ( $best->{'MAPVIL_X'}, $best->{'MAPVIL_Y'} );
}

1;
